/* MSGSPLIT.C                                                Turbo C++ v3.0
   Written 11 December 1995 by Robert B. Clark                Revision 1.00
   Copyright (c) 1995 by Robert B. Clark.  Freeware.
   
   MessageSplit extracts individual articles and mail from a ASCII text
   file consisting of many concatenated messages.

   Extracted messages will be named basename.001, basename.002, etc.
   Existing files will be overwritten without notification.

   Returns errorlevel 0 if an error occurred; otherwise returns the number
   of extracted message files.

   Disclaimer:
   
   MessageSplit is freeware and no charges may be levied for this program
   other than a nominal media or access fee.  The author assumes no
   liability for the use or applicability  of this software.  By using this
   program, the user explicitly assumes all risk and responsibility.
   
*/
#include "msgsplit.h"


int main(int argc, char *argv[])
{
   int i;
   static char inspec[MAXPATH],     /* Input file */
               outspec[MAXPATH];    /* Extracted filename(s) */

/* Process command line */
   if(ParseCmdline(argc,argv,inspec,outspec)==NULL ||
      GetSeparator(argv[0])==NULL)
      i=EXIT_FAILURE;
   else {
      DisplayOptions();
      XlateEsc(separator);    /* Translate any escape sequences */
      i=ProcessInputFile(inspec,outspec);
   }
   printf("\n\nExit status=%d.",i);
   return i;
}
/**********************************************************************/

void Raspberry(void) { sound(40);  delay(250); nosound(); }

void ErrMsg(char * msg)
/* Displays error message 'msg' to stderr device and sounds raspberry. */
{
   char *p;

   Raspberry();
   if ((p=(char *)malloc(strlen(msg)+6))==NULL)
      fprintf(stderr,msg);
   else {
      sprintf(p,"\n--> %s",msg);
      fprintf(stderr,p);
      free(p);
   }
} /* ErrMsg */


char * NormalizeFilespec(char * fspec, char * dname, char *dext)
/* Normalizes input filespec 'fspec' and sets the global 'filepath' */
/* to the path portion of the complete file specifier.              */
{
   struct ffblk f;
   char fdrv[MAXDRIVE],
        fpath[MAXPATH],
        fname[MAXFILE],
        fext[MAXEXT];
   const char defname[]="????????",
              defext[]=".???";
   int i;

/* Resolve ambiguous directory filespec */
   fnsplit(fspec,NULL,NULL,fpath,fext);   /* Get entered name.ext */
   strcat(fpath,fext);
   i=(findfirst(fspec,&f,FA_DIREC)==0);
   printf("\n\nEntry: fspec=\"%s\"",fspec);
   while (i) {
      printf("\n   name.ext=\"%s\"",f.ff_name);
      if ((f.ff_attrib & FA_DIREC) &&     /* Found a directory ...      */
         fspec[strlen(fspec)-1]!='\\' &&  /* User didn't append "\" ... */
         strcmpi(f.ff_name,fpath)==0)     /* And the name.ext matches,  */
      {                                   /* So add the slash for the   */
         strcat(fspec,"\\"); break; }     /* lazy bastard. :-)          */
     i=findnext(&f)==0;
   }
/* Default and normalize file specifier (wildcards ok) */
   i=fnsplit(fspec,fdrv,fpath,fname,fext);
   if (dname==NULL) dname=(char *)defname;
   if (dext==NULL) dext=(char *)defext;
   if ((i & FILENAME )==0) strcpy(fname,dname); /* Default filename */
   if ((i & EXTENSION)==0) strcpy(fext,dext);   /* Default extension */
   fnmerge(fspec,fdrv,fpath,fname,fext);
   if ((_fullpath(fpath,fspec,MAXPATH))!=NULL)  /* Normalized full fspec */
      strcpy(fspec,strupr(fpath));
   fnsplit(fspec,fdrv,fpath,NULL,NULL);         /* Normalized file path */
   genfilepath[0]='\0';
   fnmerge(genfilepath,fdrv,fpath,NULL,NULL);
   printf("\n\nExit: fspec=\"%s\"",fspec);
   return fspec;
} /* NormalizeFileSpec */

#ifdef foo
char * NormalizeFilespec(char * filespec)
/* Normalizes input filespec 'fspec' and sets the global 'filepath' */
/* to the path portion of the complete file specifier.              */
{
   struct ffblk f;
   char fdrv[MAXDRIVE],
        fpath[MAXPATH],
        fname[MAXFILE],
        fext[MAXEXT],
        fspec[MAXPATH];    /* Corrected filespec */
   int i,slen;

/* Resolve ambiguous directory filespec */
   fnsplit(filespec,fdrv,fpath,fspec,fext); /* Get name.ext from filespec */
   strcat(fspec,fext);                     /* Put name.ext in fspec */
   printf("\n\nEntry: filespec=\"%s\"\tfspec=\"%s\"",filespec,fspec);
   slen=strlen(filespec);
   i=(findfirst(filespec,&f,FA_DIREC)==0);
   while (i) {
      printf("\n   name.ext=\"%s\"",f.ff_name);
      if ((f.ff_attrib & FA_DIREC) &&   /* Found a directory ...      */
         filespec[slen-1]!='\\' &&      /* User didn't append "\" ... */
         strcmpi(f.ff_name,fspec)==0)   /* And the name.ext matches,  */
      {                                 /* So add the slash for the   */
         strcat(fspec,"\\");            /* lazy bastard. :-)          */
         strcat(fpath,fspec);
         fnmerge(fspec,fdrv,fpath,NULL,NULL);
         break;
      }
      i=findnext(&f)==0;
   }
   if(!i) fnmerge(fspec,fdrv,fpath,fspec,fext);
   printf("\nExit: filespec=\"%s\"\tfspec=\"%s\"",filespec,fspec);
/* Default and normalize file specifier (wildcards ok) */
   i=fnsplit(fspec,fdrv,fpath,fname,fext);
   if ((i & FILENAME )==0) strcpy(fname,"*");   /* Default filename */
   if ((i & EXTENSION)==0) strcpy(fext,".TXT"); /* Default extension */
   fnmerge(fspec,fdrv,fpath,fname,fext);
   if ((_fullpath(fpath,fspec,MAXPATH))!=NULL)  /* Normalized full fspec */
      strcpy(fspec,strupr(fpath));
   fnsplit(fspec,fdrv,fpath,NULL,NULL);         /* Normalized file path */
   genfilepath[0]='\0';
   fnmerge(genfilepath,fdrv,fpath,NULL,NULL);
   return fspec;
} /* NormalizeFileSpec */
#endif

int CheckSwitch(char *arg, char c)
/* Returns nonzero if char c immediately follows a '/' or '-' char */
{
   int x;
   char sw[2];

   sw[0]='/'; sw[1]=c;
   x=strnicmp(arg,sw,2)==0; sw[0]='-';
   return (x || strnicmp(arg,sw,2)==0);
} /* CheckSwitch */


char * ParseCmdline(int argc, char *argv[], char *fspec, char *basespec)
/* Parse command line and set appropriate globals. */
/* Return pointer to filespec or NULL if error */
{
   int i;
   char drive[MAXDRIVE],   /* Buffers to hold filespec components */
        path[MAXDIR],
        name[MAXFILE];

   fspec[0]='\0'; basespec[0]='\0'; separator[0]='\0'; msg_num=0;
   for (i=1; i<argc; i++) {
      if (CheckSwitch(argv[i],'?') || CheckSwitch(argv[i],'H')) {
         puts(Usage);                        /* Display help */
         exit(EXIT_FAILURE);                 /* and terminate */
      }
      else if (CheckSwitch(argv[i],'N'))
         msg_num=abs(atoi(argv[i]+2));       /* Message number */
      else if (CheckSwitch(argv[i],'S')) {   /* Separator line */
         strcpy(separator,argv[i+1]); i++;
         saveformat=1;                       /* Precedes msg */
      }
      else if (CheckSwitch(argv[i],'F'))     /* Force SaveArticleFormat */
      {                                      /* 0=UNIX 1=Sep 2=No sep */
         saveformat=abs(atoi(argv[i]+2));
         switch(saveformat) {
            case 0: strcpy(separator,"From "); break;
            case 1: strcpy(separator,"========\\n"); break;
            default: strcpy(separator,"From: ");
         }
      }
      else if (*argv[i]!='/' && *argv[i]!='-')  /* Not a switch:filename */
         if(!fspec[0]) strcpy(fspec,argv[i]);   /* 1st filespec=fspec */
         else if (!basespec[0])                 /* 2nd filespec=basespec */
            strcpy(basespec,argv[i]);           /* 3rd+ filespec=ignored */
   } /* for */

/* 'fspec' is NOT optional */
   if (!fspec[0]) {
      ErrMsg("You must specify an input file.");
      return NULL;
   }
   NormalizeFilespec(fspec,NULL,".TXT");

/* Default basespec from fspec if not specified */
   if (!basespec[0]) strcpy(basespec,fspec);
   i=fnsplit(basespec,drive,path,name,NULL);
   printf("\nBefore: basespec=\"%s\"\tdrive=%s\tpath=%s\tname=%s",
      basespec,drive,path,name);
   if ((i & FILENAME)==0) fnsplit(fspec,NULL,NULL,name,NULL);
   fnmerge(basespec,drive,path,name,NULL);
   printf("\nAfter: basespec=\"%s\"",basespec);
   fnsplit(fspec,NULL,NULL,name,NULL);
   NormalizeFilespec(basespec,name,NULL);
   return fspec;
} /* ParseCmdline */


int FileExists(char *filename)
{
   FILE *fp;

   fp=fopen(filename,"r");
   if (fp!=NULL) fclose(fp);
   return fp!=NULL;
} /* FileExists */


void AppendSlash(char *fspec)
{ if (fspec[strlen(fspec)]!='\\') strcat(fspec,"\\"); }


char * SearchPath(char *filespec)
/* Searches all directories in path for file filespec */
/* Returns pointer to directory name if found or NULL if not found. */
{
   char fspec[MAXPATH], *pptr, *path, *p=NULL;

/* Allocate buffer as req'd for those who insist upon >128-byte PATHs */
   if ((pptr=getenv("PATH"))!=NULL &&
      (path=(char *)malloc(strlen(pptr)+1))!=NULL)
   {
      strcpy(path,pptr);
      p=strtok(path,";");
      while(p) {
         strcpy(fspec,p); AppendSlash(fspec); strcat(fspec,filespec);
         if (FileExists(fspec)) break;
         p=strtok(NULL,";");
      }
      free(path);
   }
   return p;
} /* SearchPath */


char * GetINIValue(FILE *fp, char *token, char *value)
/* Scan INI file for 'token', which is case-sensitive and starts at */
/* column 1 in the INI file.  If found, return pointer to 'value', */
/* which includes the newline.  Otherwise return NULL. */
{
   int len, found;
   char *p;

   fseek(fp, 0L, SEEK_SET);      /* Roll back to start of file */
   len=strlen(token); found=0;   /* okay, i'm too lazy to do elev seek */
   while (!found && !feof(fp)) { /* Read lines until found or eof */
      fgets(buf,LINELEN,fp);
      found=(memcmp(buf,token,len)==0);
   }
   p=NULL;
   if (found) {
      p=strchr(buf,'=')+1;       /* Now points at either " or integer */
      if (*p=='\"') {            /* String value */
         p++;                    /* Return everything within "" incl \n */
         strncpy(value,p,strrchr(buf,'\"')-p);
      }
      else strcpy(value,p);      /* Numeric value */
      p=value;                   /* Return everything after '=' incl \n */
      if (value[strlen(value)-1]=='\n')
         value[strlen(value)-1]='\0';
   }
   return p;
} /* GetINIValue */


char * GetSeparator(char *execpath)
/* Reads info from Agent.INI file. */
/* Search method: -S option, current dir, argv[0] directory, path. */
/* Returns null if no separator, else returns pointer to separator */
{
   char drive[MAXDRIVE], curr_path[MAXPATH], execspec[MAXPATH];
   FILE *fp;

   if (separator[0]) return separator;     /* -S override was used */

/* Save "executable_path\agent.ini" in execspec */
   fnsplit(execpath,drive,curr_path,NULL,NULL);
   fnmerge(execspec,drive,curr_path,INIFNAME,INIFEXT);
   NormalizeFilespec(execspec,NULL,NULL);

/* Save "current_directory\agent.ini" in curr_path */
   getcwd(curr_path,MAXPATH);
   strcat(curr_path,"\\"INIFSPEC);

/* Search for INI file; copy found filespec to genfilepath */
   if (FileExists(curr_path))          /* Search current dir */
      strcpy(genfilepath,curr_path);
   else if (FileExists(execspec))      /* Search executable dir */
      strcpy(genfilepath,execspec);
   else {                              /* Search path directories */
      strcpy(genfilepath,SearchPath(INIFSPEC));
      if (genfilepath[0])
         fnmerge(genfilepath,NULL,genfilepath,INIFNAME,INIFEXT);
   }

   if (!genfilepath[0]) {
      ErrMsg(INIFSPEC " not found!\nUse the /S switch override.");
      return NULL;                                  /* No INI file */
   }
   if ((fp=fopen(genfilepath,"r"))==NULL) {
      ErrMsg("Unable to open " INIFSPEC "!");
      return NULL;
   }
/* Check SaveArticleFormat (0=UNIX, 1=Separator, 2=No separator) */
   GetINIValue(fp,"SaveArticleFormat=",buf);
   saveformat=atoi(buf);
   switch (saveformat) {
      case 0:  strcpy(separator,"From "); break;
      case 1:  if (GetINIValue(fp,"SaveArticleSeparator",separator)!=NULL)
                  break;
      case 2:
      default: strcpy(separator,"From: ");
   }
   fclose(fp);
   return genfilepath;
} /* GetSeparator */


void XlateEsc(char *buf)
/* Replaces LITERAL escape char markers in buf with single-char */
/* equivalents. Simply could not do this with sprintf or sscanf */
{
   char *p,*p0;

   p=buf;
   while(*p!='\0') {
      p0=strchr(buf,0);       /* Find end of string */
      if (*p=='\\') {
         switch(*(p+1)) {
            case('t'):  *p='\t'; break;   /* Tab */
            case('n'):  *p='\n'; break;   /* Newline */
            case('b'):  *p='\b'; break;   /* Backspace */
            case('f'):  *p='\f'; break;   /* Form feed */
            case('a'):  *p='\a'; break;   /* Bell */
            case('r'):  *p='\r'; break;   /* Carriage return */
            case('\\'): *p='\\'; break;   /* Literal backslash */
            case('\"'): *p='\"'; break;   /* Literal double quote */
            case('\''): *p='\''; break;   /* Literal single quote */
         }
         memmove(p+1,p+2,p0-(p+2)+1);     /* Delete char after "\" */
      }
      ++p;
   }
} /* XlateEsc */


int IsSepLine(char *buf)
/* Determine if line starts with separator string */
/* If using "From" header, check for '@' in "From[:] user@domain.net" */
{
   int issep;

   issep=(memcmp(buf,separator,strlen(separator))==0);
   if ((saveformat==0 || saveformat==2) && issep)
      issep=(strchr(buf,'@')!=NULL);
   return issep;
} /* IsSepLine */


int ProcessInputFile(char *inspec, char *outspec)
/* Run through inspec and write each msg to outspec+incremented extension */
/* If msg_num is > 0, write only that numbered message. */
/* Returns number of outfiles written.  */
{
   FILE *inf, *outf;
   int ctr=0, done=0, out_ctr=0, issep;
   char outfile[MAXPATH], fname[MAXFILE], fpath[MAXDIR];
   const char errfmt[]=" : %s";

/* Put outspec d:\path\name into genfilepath */
   fnsplit(outspec,outfile,fpath,fname,NULL);      /* Put d:\path\name */
   fnmerge(genfilepath,outfile,fpath,fname,NULL);  /* into genfilepath */

   if ((inf=fopen(inspec,"r"))==NULL)
   {
      sprintf(outfile,"Could not open %s",inspec);
      ErrMsg(outfile);
      return 0;
   }
   printf("\nExtracting message%s from %s ...",
      msg_num==0?"s":(sprintf(buf," #%d",msg_num),buf),inspec);

/* Read lines from inspec until done or eof */
   while (!done && (fgets(buf,LINELEN,inf)!=NULL))
   {
      issep=IsSepLine(buf);
      while (issep && !done)
      {
         ++ctr;
         if (ctr==msg_num || msg_num==0) {   /* Open a new outfile */
            sprintf(outfile,"%s.%.3d",genfilepath,ctr);
            printf("\n   Writing %s",outfile);

            if ((outf=fopen(outfile,"w"))==NULL) {
               fprintf(stderr,errfmt,sys_errlist[errno]);
               done=10;       /* Don't display addtl error message */
            }
            else if (fputs(buf,outf)!=EOF)
               out_ctr++;
         }

/*       Read/write until next separator line or EOF */
         issep=0;
         while (!issep && (fgets(buf,LINELEN,inf)!=NULL) && !done)
         {
            issep=IsSepLine(buf);
            if (!issep && outf && !ferror(outf))
               if (fputs(buf,outf)==EOF)
               {
                  fprintf(stderr,errfmt,sys_errlist[errno]);
                  fclose(outf);
                  remove(outfile);     /* Delete short file */
                  done=10;             /* Don't display addtl */
                  out_ctr--;           /* error message */
               }
         }
         fclose(outf);
         if (!done && ((ctr==msg_num) || feof(inf))) done=1;
      } /* while issep & not done */
   } /* while !done & not eof */

   if (!out_ctr && done!=10)          /* No messages were extracted */
   {
      if (msg_num)
         sprintf(fpath,"Could not find message #%d in %s.",
            msg_num,inspec);
      else
         sprintf(fpath,"Missing separator lines in %s.",inspec);
      ErrMsg(fpath);
   }
   fcloseall();
   return out_ctr;
} /* ProcessInputFile */


void DisplayOptions(void)
/* Displays current settings */
{
   char msg[40];

   puts(CREDIT);
   switch (saveformat) {
      case 0:  strcpy(msg,"UNIX message file"); break;
      case 2:  strcpy(msg,"No separators"); break;
      case 1:
      default: strcpy(msg,"Precede message with separator");
   }
   printf("\n   \xfe Using %s",
      strstr(genfilepath,INIFSPEC)!=NULL ? genfilepath :
      "command line parameters");
   printf("\n   \xfe SaveArticleSeparator=\"%s\"",separator);
   printf("\n   \xfe SaveArticleFormat=%d  (%s)\n",saveformat,msg);
} /* DisplayOptions */
