
/**********************************************************************/
/* MSGSPLIT.H - Turbo C header definitions for MessageSplit v1.00     */
/* Written 11 December 1995 by Robert B. Clark                        */
/* This program is freeware.                                          */
/**********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <io.h>               /* For lseek() */
#include <dos.h>              /* For FA* definitions & DOS functs */
#include <dir.h>              /* For MAX* definitions */
#include <string.h>           /* For str*() functions */


#define PROGNAME        "MSGSPLIT"
#define VERSION         "1.00"
#define INIFNAME        "AGENT"
#define INIFEXT         ".INI"
#define INIFSPEC        INIFNAME INIFEXT
#define CREDIT          "\n" PROGNAME " v" VERSION " "         \
                        "(Compiled "__DATE__" " __TIME__ ")"   \
                        "\nCopyright (c) 1995 by "             \
                        "Robert B. Clark <rclark@iquest.net>"  \
                        "\nThis program is freeware."

#ifdef  EXIT_SUCCESS       /* In this program, zero means error */
   #undef  EXIT_SUCCESS
   #undef  EXIT_FAILURE
#endif

#define EXIT_SUCCESS    1
#define EXIT_FAILURE    0
#define LINELEN         256   /* Buffer line length=255 + newline */
/*************************************************************************/
/* Globals */

char genfilepath[MAXPATH],    /* Generic file specifier buffer */
     buf[LINELEN],            /* Generic file I/O buffer */
     separator[LINELEN];      /* SaveArticleSeparator string */
int  msg_num,                 /* Message number to extract  (0=all) */
     saveformat;              /* SaveArticleFormat (0..2; >2==2) */

const char Usage[] =
"MessageSplit splits a concatenated ASCII file into individual "
"message files."
"\n\nUsage: " PROGNAME " [drive:][path]filename[.txt] [basename] "
"[/Nmsg_num]\n\t[/S\"separator\"] [/?|/H]\n\n"
"  filename\tName of the concatenated ASCII file.\n"
"\t\tThe extension defaults to .TXT if not specified.\n"
"  basename\tBase filename of the extracted messages file(s).\n"
"\t\tThe drive and path may be included if desired.\n"
"\t\tIf not specified, defaults from the \"filename\" parameter.\n"
"  /Nmsg_num\tExtracts only the msg_num'th message out of the file.\n"
"  /S\"sep\"\tCase-sensitive separator string.  Mainly for use with\n"
" \t\tnon-Agent newsreaders and to override " INIFSPEC ".\n\n"
"Extracted messages will be named basename.001, basename.002, etc.\n"
"Existing files will be overwritten without notification.\n"
"Returns the number of extracted messages as an errorlevel.\n"
CREDIT;
/*************************************************************************/
/* Prototypes */

void  Raspberry(void);
void  ErrMsg(char * msg);
char *NormalizeFilespec(char * fspec, char *dname, char *dext);
int   CheckSwitch(char *arg, char c);
char *ParseCmdline(int argc, char *argv[], char *fspec, char *basespec);
int   FileExists(char *filename);
void  AppendSlash(char *fspec);
char *SearchPath(char *filespec);
char *GetINIValue(FILE *fp, char *token, char *value);
char *GetSeparator(char *execpath);
void  XlateEsc(char *buf);
int   IsSepLine(char *buf);
int   ProcessInputFile(char *inspec, char *outspec);
void  DisplayOptions(void);
/***********************  end of definitions  ***************************/
